% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation, version 2.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software Foundation,
% Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
%


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%     Converts the a,b parameters of an       %
%  ellipse/hyperbola to the p,d parameters    %
%                                             %
%   Created : 10/02/2005                      %
%    Author : Christopher Mei                 %
%                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%% Equations %%%%%%%%%%%%%%%%%%%%
%Paraboloid : x^2+y^2=2p(z+2p) (type==1)
%Hyperboloid : (z+d/2)^2/a^2-x^2/b^2-y^2/b^2=1 (type==2)
%Ellipsoid : (z+d/2)^2/a^2+x^2/b^2+y^2/b^2=1 (type==3)

%%%%%%%%%%%%%%%%%% Input %%%%%%%%%%%%%%%%%%%%%%%
% a,b : parameters of the conic
%type : Hyperboloid (2)/Ellipsoid (3)

function [p,d] = ab2pd(a, b, type)

p = b^2/(2*a);

if type == 1
  d = 1;
end
if type == 2
  d = 2*sqrt(a^2+b^2);
end
if type == 3
  d = 2*sqrt(a^2-b^2);
end
