% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation, version 2.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software Foundation,
% Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
%


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%     Fits a circle to the mirror border to   %
%        initialise intrinsic values          %
%                                             %
%   Created : 2005                            %
%    Author : Christopher Mei                 %
%                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

var2fix = 'map';

fixvariable;

if ~exist('n_ima'),
    data_calib;
end;

if ~exist('mirror_diam'),
  disp('Please set the value of ''mirror_diam'' in ''Set mirror values''');
  return;
end;

check_active_images;

if ~exist(['I_' num2str(ind_active(1))]),
    ima_read_calib;
    if isempty(ind_read),
        disp('Cannot extract border without images');
        return;
    end;
end;

fprintf(1,'\nExtraction of the mirror border from the images\n');

if (exist('map')~=1), map = gray(256); end;


ima_proc = 1:n_ima;

listImages={};

for kk = ima_proc,
  name = ['I_' num2str(kk)];
  if exist(name)
    listImages(length(listImages)+1) =  { name };
  end
end

fprintf(1,'Calculating min over all images... ');
minImagesDouble;
fprintf(1,'done.\n');

% $$$ figure(2);				% 
% $$$ imshow(uint8(I));
% $$$ 
% $$$ fprintf(1,'Please click on the mirror center.\n');
% $$$ CR=digipts3(1);
% $$$ 
% $$$ [center_estimate,radius_estimate]=extractCircleAuto(I,CR(:,1));
% $$$ 
% $$$ draw2DCircle(center_estimate,radius_estimate,'r-');
% $$$ plot(center_estimate(1),center_estimate(2),'yx','MarkerSize',10);
% $$$ 
% $$$ correctExtr = input('Was the extraction successful ? ([]=yes, other=no) : ','s');
% $$$ 
% $$$ if isempty(correctExtr)
% $$$   disp('Calculating an estimate for the intrinsic parameters.');
% $$$ else

figure(2);
imshow(uint8(I));
fprintf(1,['Please click on the mirror center and then on the' ...
	   ' mirror inner border.\n']);
CR=digipts3(2);
[center_estimate,radius_estimate]=extractCircle(I, CR(:,1),CR(:,2));
draw2DCircle(center_estimate,radius_estimate,'r-');
plot(center_estimate(1),center_estimate(2),'yx');

correctExtr = input('Was the extraction successful ? ([]=yes, other=no) : ','s');

if ~isempty(correctExtr)
  disp(['You will have to set ''center_estimate'' and ''radius_estimate''' ...
	' manually.']);
end
%end

T_estimate=[center_estimate;1]

%if ~exist('mirror_diam')|mirror_diam<0
%  disp(['The mirror diameter is unknown, we will not estimate the' ...
%	' magnifying factor']);
%  F_estimate=1;
%  KK_estimate=[1,0,T_estimate(1);...
%	       0,1,T_estimate(2);...
%	       0,0,1];
%else

if mirror_type==1
  F_estimate=radius_estimate/(mirror_diam/2)
else
  F_estimate=mirror_depth*radius_estimate/(mirror_diam/2);
end

KK_estimate=[F_estimate,0,T_estimate(1);...
	     0,F_estimate,T_estimate(2);...
	     0,0,1]

%Draw border points
if mirror_type==1
  B1=KK_estimate*[mirror_diam/2+T_estimate(1);0;1];
  B2=KK_estimate*[-mirror_diam/2+T_estimate(1);0;1];
  B3=KK_estimate*[0;mirror_diam+T_estimate(1);1];
  B4=KK_estimate*[0;-mirror_diam+T_estimate(1);1];
else
  B1=KK_estimate*[mirror_diam/(2*mirror_depth);0;1];
  B2=KK_estimate*[-mirror_diam/(2*mirror_depth);0;1];
  B3=KK_estimate*[0;mirror_diam/(2*mirror_depth);1];
  B4=KK_estimate*[0;-mirror_diam/(2*mirror_depth);1];
end

plot(B1(1),B1(2),'g+');
plot(B2(1),B2(2),'m+');
plot(B3(1),B3(2),'g+');
plot(B4(1),B4(2),'m+');


%Draw center
C1=KK_estimate*[0;0;1];
plot(C1(1),C1(2),'y+','MarkerSize',10);
