% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation, version 2.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software Foundation,
% Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Unified PnP solving algorithm based on       %
% the method by Lu&al                          %
%                                              %
%  Created : 06/04/2005                        %
%   Author : Christopher Mei                   %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%% Input %%%%%%%%%%%%%%%%%%%%%%%
%points3D : points of a know 3D structure in
%           a global frame (ex : grid, cube,...)
%points2D : points seen in the image plane

%%%%%%%%%%%%%%%%% Output %%%%%%%%%%%%%%%%%%%%%%%
% Q,T : quaternion rotation and translation between the
%       two frames.
% V : estimate of the distortion and camera intrinsic parameters
% error : image-space error associated with the estimate
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [Q,T, error]=fastOmniPnPSphere(points3D, points2D, V, p, ...
					d, type, Rinit)


% Lift points back to sphere
points2D_lift=fullLiftingSphere(points2D, V, p, d, type);
points2D_lift=points2D_lift*inv(diag(points2D_lift(3,:)));

% Use Lu&al algorithm to estimate pose
options.method = 'QTN';
options.tol = 1e-6;
options.epsilon = 1e-8;

if nargin==7
  options.initR=Rinit;
end

[R, T, it, obj_err, error] = objpose(points3D,points2D_lift(1:2,:),options);

% Switch back to direct transformation
if (det(R)<0)
  R=-R;
end
Q=mat2quat(R);
T=-T;

