% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation, version 2.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software Foundation,
% Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
%


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Unified projection model for omnidirectional %
%                  sensor                      %
%                                              %
%  Created : 02/09/2004                        %
%   Author : Christopher Mei                   %
% Based on : Barreto PhD 2003                  %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%% Input %%%%%%%%%%%%%%%%%%%%%%%
% Xh : points to project, 3xN matrix
% p : latus rectum
% d : distance between focal points (any for paraboloid)
% KK : projection matrix
% type : Perspective (0)/Paraboloid (1)/Hyperboloid (2)/Ellipsoid (3)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function P = fastUnifiedPointProjectionK(Xh, p, d, KK, type)

nb=size(Xh);
nb=nb(2);

%Conic parameters
if type==0
  xi = 0;
  xiMphi = -1;
end
if type==1 | type == 4
  xi=1;
  xiMphi=-2*p;
end
if type==2
  denom=sqrt(d^2+4*p^2);
  xi=d/denom;
  xiMphi=(-2*p)/denom;
end
if type==3
  denom=sqrt(d^2+4*p^2);
  xi=d/denom;
  xiMphi=(2*p)/denom;
end

P=zeros(3,nb);

for col=1:nb
  x=Xh(1,col);
  y=Xh(2,col);
  z=Xh(3,col);

  P(1,col)=x;
  P(2,col)=y;
  P(3,col)=(z-xi*sqrt(x^2+y^2+z^2))/xiMphi;
end

P=KK*P;

P = P(1:2,:) ./ (ones(2,1)*P(3,:));

