% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation, version 2.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software Foundation,
% Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This function calculates the focals to obtain the same size %
% circle after removing the distortion.                       %
%                                                             %
%   Created : 2005                                            %
%    Author : Christopher Mei                                 %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [cf1,cf2, roi_min, roi_max]=calculateFocalsUnbiased(center,radius,kc,alpha_c,fc,cc)
plotValues=0;

V=[kc;alpha_c;fc;cc];

border_right=center+[radius;0];
border_left=center+[-radius;0];
border_top=center+[0;-radius];
border_bottom=center+[0;radius];

if plotValues
  plot(center(1),center(2),'g+');
  plot(border_right(1),border_right(2),'g+');
  plot(border_left(1),border_left(2),'g+');
  plot(border_top(1),border_top(2),'g+');
  plot(border_bottom(1),border_bottom(2),'g+');
end

center_lift=partialLiftingSphere(center,V);

border_right_lift = partialLiftingSphere(border_right,V);
border_left_lift = partialLiftingSphere(border_left,V);
border_top_lift = partialLiftingSphere(border_top,V);
border_bottom_lift = partialLiftingSphere(border_bottom,V);

reproj_center=partialProjectionSphere(center_lift, V);

reproj_border_right = partialProjectionSphere(border_right_lift,V);
reproj_border_left = partialProjectionSphere(border_left_lift,V);
reproj_border_top = partialProjectionSphere(border_top_lift,V);
reproj_border_bottom = partialProjectionSphere(border_bottom_lift,V);

if plotValues
  plot(reproj_center(1),reproj_center(2),'yx');
  plot(reproj_border_right(1),reproj_border_right(2),'yx');
  plot(reproj_border_left(1),reproj_border_left(2),'yx');
  plot(reproj_border_top(1),reproj_border_top(2),'yx');
  plot(reproj_border_bottom(1),reproj_border_bottom(2),'yx');
end

KK=makeKK(V(6:10));
invKK=inv(KK);

cc = invKK(1:2,3);
fc = [invKK(1,1);invKK(2,2)];
alpha_c = invKK(1,2)/fc(1);

new_f1_right = (border_right(1)-cc(1))/border_right_lift(1);
new_f1_left = (border_left(1)-cc(1))/border_left_lift(1);
new_f2_top=(border_top(2)-cc(2))/border_top_lift(2);
new_f2_bottom=(border_bottom(2)-cc(2))/border_bottom_lift(2);

cf1=max(new_f1_right,new_f1_left);
cf2=max(new_f2_top,new_f2_bottom);

testKK=makeKK([0;cf1;cf2;cc]);

reproj_border_right = testKK*[border_right_lift;1];
reproj_border_left = testKK*[border_left_lift;1];
reproj_border_top = testKK*[border_top_lift;1];
reproj_border_bottom = testKK*[border_bottom_lift;1];

if plotValues
  plot(reproj_center(1),reproj_center(2),'bo');
  plot(reproj_border_right(1),reproj_border_right(2),'bo');
  plot(reproj_border_left(1),reproj_border_left(2),'bo');
  plot(reproj_border_top(1),reproj_border_top(2),'bo');
  plot(reproj_border_bottom(1),reproj_border_bottom(2),'bo');
end

roi_min=[reproj_border_left(1);reproj_border_bottom(2)];
roi_max=[reproj_border_right(1);reproj_border_top(2)];

if plotValues
  plot(roi_min(1),roi_min(2),'rx');
  plot(roi_max(1),roi_max(2),'bx');
end
