% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation, version 2.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software Foundation,
% Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                 %
%     Calculates the scalar product function      %
%               and its jacobian                  %
%                                                 %
%   Created : 2005                                %
%    Author : Christopher Mei                     %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [P,dPdX,dPdY] = minScalarProduct(X, Y)

if nargin==0
  disp('Launching test...');
  test;
  return
end
switch nargout
 case 0
 case 1
  P=operation(X,Y);
 case 2
  P=operation(X,Y);
  dPdX = jacobianX(X,Y);
 case 3
   P=operation(X,Y);
   dPdX = jacobianX(X,Y);
   dPdY = jacobianY(X,Y);
 otherwise
  disp('Too many output args.')
end

function test
error=1e-4;
disp(['Error : 1e' slog10(error)]);

%%% Values %%%
X=randn(2,1);
dX=randn(2,1)*error;

Y=randn(2,1);
dY=randn(2,1)*error;

[P,dPdX,dPdY]=minScalarProduct(X,Y);

%%% Test derivation in X %%%
Pp=minScalarProduct(X+dX,Y);

gainP=norm(Pp-P)/norm(Pp-P-dPdX*dX);
disp(['Estimate gain in X : 1e' slog10(gainP)]);

%%% Test derivation in Y %%%
Pp=minScalarProduct(X,Y+dY);

gainP=norm(Pp-P)/norm(Pp-P-dPdY*dY);
disp(['Estimate gain in Y : 1e' slog10(gainP)]);

function P=operation(X,Y)
%P=-X'*Y;
%To obtain [X1'*Y1 X2'*Y2 ... Xn'*Yn]
P=-sum(X.*Y);

function dPdX = jacobianX(X,Y)
dPdX=-Y';

function dPdY = jacobianY(X,Y)
dPdY=-X';

function s=slog10(val)
s=num2str(floor(log10(val)));
