% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation, version 2.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software Foundation,
% Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                             %
%   Writes a Look-Up Table (LUT) to a file    %
%  to undistort central catadioptric images   %
%  in the 'unbiased' calibration case         %
%                                             %
%   Created : 2005                            %
%    Author : Christopher Mei                 %
%                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function write_lut_unbiased(width,height,k,alpha,f,c,KK_new)

KK_old=makeKK([alpha;f;c]);

invKK_new=inv(KK_new);

width_ones=ones(1,width);

fidx = fopen('lutx_sphere','w');
fidy = fopen('luty_sphere','w');

fidx_floor = fopen('lutx_floor_sphere','w');
fidy_floor = fopen('luty_floor_sphere','w');

fidx_diff = fopen('lutx_diff_sphere','w');
fidy_diff = fopen('luty_diff_sphere','w');

fidx_round = fopen('lutx_round_sphere','w');
fidy_round = fopen('luty_round_sphere','w');

writeHeaders(fidx,fidy,width,height);
writeHeaders(fidx_floor,fidy_floor,width,height);
writeHeaders(fidx_diff,fidy_diff,width,height);
writeHeaders(fidx_round,fidy_round,width,height);

nbNotChanged=0;
nbChanged=0;

fprintf('\n');

for i=1:height
  fprintf('\r%i/%i',i,height);
  P=[1:width;
     i*width_ones;
     width_ones];

  P = invKK_new*P;
  P = (P(1:2,:)/P(3));
  P = KK_old*[undistort(k,P);width_ones];
  
  %Set the incorrect values to -1
  indX=find(P(1,:)<1|P(1,:)>width);
  indY=find(P(2,:)<1|P(2,:)>height);
  
  P(1:2,indX)=1*ones(2, size(indX,2));
  P(1:2,indY)=1*ones(2, size(indY,2));
  
  %Set the incorrect values to -1
  %indC=sqrt(sum(P.^2,1))
  
  %Coordinates in C start in 0
  P = P-1;
  
  %return
  writeRows(P,fidx,fidy);
  writeRowsFloor(P,fidx_floor,fidy_floor);
  writeRowsDiff(P,fidx_diff,fidy_diff);
  writeRowsRound(P,fidx_round,fidy_round);

  for j=1:width
    if (round(P(1,j))==j-1)&(round(P(2,j))==i-1)
      nbNotChanged=nbNotChanged+1;
    else
      nbChanged=nbChanged+1;
    end
  end
end

%total = nbChanged+nbNotChanged
%disp(['Total not changed : ' num2str((nbNotChanged/total)*100) '%']);
%disp(['Total changed : ' num2str((nbChanged/total)*100) '%']);

fclose(fidx_floor);
fclose(fidy_floor);
fclose(fidx_diff);
fclose(fidy_diff);
fclose(fidx_round);
fclose(fidy_round);

function writeHeaders(fidx,fidy,width,height)
fwrite(fidx,width,'integer*4');
fwrite(fidx,height,'integer*4');
fwrite(fidy,width,'integer*4');
fwrite(fidy,height,'integer*4');

function writeRows(P,fidx,fidy)
fwrite(fidx,P(2,:),'float');
fwrite(fidy,P(1,:),'float');

function writeRowsFloor(P,fidx_floor,fidy_floor)
fwrite(fidx_floor,floor(P(2,:)),'int');
fwrite(fidy_floor,floor(P(1,:)),'int');

function writeRowsDiff(P,fidx_diff, fidy_diff)
floorP1=floor(P(1,:));
floorP2=floor(P(2,:));

fwrite(fidx_diff,P(2,:)-floorP2,'float');
fwrite(fidy_diff,P(1,:)-floorP1,'float');

function writeRowsRound(P,fidx,fidy)
fwrite(fidx,round(P(2,:)),'int');
fwrite(fidy,round(P(1,:)),'int');

